function [V, D] = clustEigs(A,prtc,k,str)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of a symmetric matrix A.
% 
% [V, D] = clustEigs(A,prtc,k,str);
%
% Input:
% A     - Sparse symmetric matrix
% prtc  - Cell-array containing vertices for each cluster
% k     - Scalar or vector indicating rank of cluster approximations.
%         If k is scalar then the same k is used for all clusters. 
%         If k is vector, then length(k) must equal number of clusters.
% str   - string, must be "eigs" or "laneig" to indicate altorithm
%
% Output:
% V     - Cell-array with top k eigenvectors of each cluster.
% D     - Cell-array with top k eigenvalues of each cluster.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of clusters
noc = length(prtc);     

% Initiate output variables 
V = cell(noc,1); 
D = cell(noc,1);

if length(k) == 1;
    k = ones(noc,1)*k;
end

switch str
    case 'eigs'
        for i = 1:noc
            ind = prtc{i};
            eigsOpt.disp = 0;
            disp(['Low rank computation (eigs) for cluster ' int2str(i), ' of ' int2str(noc)])
            [V{i}, D{i}] = eigs(A(ind,ind),k(i),'LM',eigsOpt);
        end
    case 'laneig'
        for i = 1:noc
            ind = prtc{i};
            disp(['Low rank computation (laneig) for cluster ' int2str(i), ' of ' int2str(noc)])
            [V{i}, D{i}] = laneig(A(ind,ind),k(i),'LM');
        end
    otherwise 
        disp('Valid string options are "eigs" and "laneig".')
end

