function [nnzBlks, nnzBlFr, nnzFrac] = compClustNnz(A,prtc)
%--------------------------------------------------------------------------
% Compute number and fraction of non-zeros in blocks of a square matrix A.
% 
% [nnzBlks, nnzBlFr, nnzFrac] = compClustNnz(A,prtc);
%
% Input:
% A     - Sparse square matrix 
% prtc  - Cell-array containing vertices for each cluster
%
% Output:
% nnzBlks   - number of nonzeros for each block A_{ij}
% nnzBlFr   - fraction of nonzeros for each block A_{ij}
% nnzFrac   - fraction of nonzeros within all diagonal blocks
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

nnzA = nnz(A);
noc = length(prtc);
nnzBlks = zeros(noc,noc);

for i = 1:noc
    for j = 1:noc
        nnzBlks(i,j) = nnz(A(prtc{i},prtc{j}));      
    end
end

nnzFrac = sum(diag(nnzBlks))/nnzA*100;
nnzBlFr = nnzBlks/nnzA*100;
