function [nnzBlks, nnzBlFr] = compClustNnzRect(A,rprtc,cprtc)
%--------------------------------------------------------------------------
% Compute number and fraction of non-zeros in blocks of a rectangular
% matrix A.
%
% [nnzBlks, nnzBlFr] = compClustNnzRect(A,rprtc,cprtc);
%
% Input:
% A     - Sparse rectangular matrix
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
%
% Output:
% nnzBlks   - number of nonzeros for each block Aij
% nnzBlFr   - fraction of nonzeros for each block Aij
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

nnzA = nnz(A);
rnoc = length(rprtc);
cnoc = length(cprtc);

nnzBlks = zeros(rnoc,cnoc);

for i = 1:rnoc
    for j = 1:cnoc
        nnzBlks(i,j) = nnz(A(rprtc{i},cprtc{j}));        
    end
end
nnzBlFr = nnzBlks/nnzA*100;

