function relerr = compClustRelErr(A,prtc)
%--------------------------------------------------------------------------
% Compute the relative error in a blockdiagonal approximation
% || A - diag(A_{11}, ..., A_{cc}) ||_F / || A ||_F * 100
% 
% relerr = compClustRelErr(A,prtc)
%
% Input:
% A     - adjacency matrix of the graph
% prtc  - cell-array with each cell containing the vertices for one
%         cluster.
%
% Output:
% relerr    - computed relative error in %
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

nA = norm(A,'fro');
noc = length(prtc);
nb = 0;
for i = 1:noc
    ind = prtc{i};
    nb = nb + norm(A(ind,ind),'fro')^2; 
end

relerr = sqrt(nA^2 - nb)/nA*100;
