function relerr = compClustRelErrExt(A,rprtc,cprtc,msk)
%--------------------------------------------------------------------------
% Compute the relative error in a dense block approximation
% || A - A_dense ||_F / || A ||_F * 100,  where A_dense only contains
% non-zeros from dense blocks
%
% relerr = compClustRelErrExt(A,rprtc,cprtc,msk)
%
% Input:
% A     - adjacency matrix of the graph
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
% msk   - Binary mask matrix indicating dense blocks
%
% Output:
% relerr    - computed relative error in %
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

nA = norm(A,'fro');
rnoc = length(rprtc);
cnoc = length(cprtc);
nb = 0;
for i = 1:rnoc
    for j = 1:cnoc
        if msk(i,j)
            rind = rprtc{i};
            cind = cprtc{j};
            nb = nb + norm(A(rind,cind),'fro')^2;
        end
    end
end

relerr = sqrt(nA^2 - nb)/nA*100;
