function SS = compCoreNonsymCell(A,U,S,V,rprtc,cprtc,str)
%--------------------------------------------------------------------------
% Compute the core/middle matrix in cell format for the clustered matrix 
% approximation.
%
% SS = compCoreNonsymCell(A,U,S,V,rprtc,cprtc,str)
% 
% Input:
% A     - Sparse square non-symmetric matrix 
% U     - Cell-array with the left  orthonormal matrices for each cluster.
% V     - Cell-array with the right orthonormal matrices for each cluster.
% S     - Optional cell-array with the top k singular values of each cluster.
%           Use [] when S is omitted.
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
% str   - string, must be 'diag' or 'dense', 
%           If str = 'diag' then diagonal blocks of SS are given by S.
%           If str = 'dense' then diagonal blocks of SS are computed.
%
% Output: 
% SS    - The core/middle matrix in cell format of the clustered matrix
%           approximation.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

rnoc = length(rprtc);
cnoc = length(cprtc);
switch str
    case 'diag'
        SS = func1(A,U,S,V,rprtc,cprtc,rnoc,cnoc);
    case 'dense'
        SS = func2(A,U,V,rprtc,cprtc,rnoc,cnoc);
    otherwise
        disp('The str argument must be "diag" or "dense".')
end

function SS = func1(A,U,S,V,rprtc,cprtc,rnoc,cnoc)
SS = cell(rnoc,cnoc);
for i = 1:rnoc
    for j = 1:cnoc
        if i == j
            SS{i,i} = S{i};
        else
            SS{i,j} = U{i}'*(A(rprtc{i},cprtc{j})*V{j});
        end
    end
end

function SS = func2(A,U,V,rprtc,cprtc,rnoc,cnoc)
SS = cell(rnoc,cnoc);
for i = 1:rnoc
    for j = 1:cnoc
         SS{i,j} = U{i}'*(A(rprtc{i},cprtc{j})*V{j});
    end
end





