function a = compSubspaceAngles(U,Uc,prtc)
%--------------------------------------------------------------------------
% Compute cosines of principal angles between range(U) and range(Uc)
% 
% a = compSubspaceAngles(U,Uc,prtc);
%
% Input:
% U     - Orthonormal matrix spanning a low dimensional column space
% Uc    - Cell-array with orthonormal matrices Ui from a clustered matrix 
%         approximation
% prtc  - Cell-array containing vertices for the partitioning of Uc
%
% Output:
% a     - Cosines of the principal angles between range(U) and range(Uc)
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of clusters
noc = length(prtc); 

m = size(U,2);
nn = zeros(noc,1);

for i = 1:noc
    nn(i) = size(Uc{i},2);
end

S = zeros(m,sum(nn));
lowInd = 1;

for i = 1:noc
    uppInd = lowInd + nn(i) -1;
    S(:,lowInd:uppInd) = U(prtc{i},:)'*Uc{i};
    lowInd = uppInd+1;
end

a = svd(S);

