function convert2MetisGraph(A,fileName)
%--------------------------------------------------------------------------
% Convert a matlab sparse matrix A and store it as a file in format 
% readable by METIS. It is assumed that A corresponds to the adjacency 
% matrix of a graph with one (connected) component.
%
% convert2MetisGraph(A,fileName)
% 
% Input:
% A         - sparse matrix 
% fileName  - string indicating the first part of the file name in which 
%             the graph is saved. Full name becomes fileName-metis-graph
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

nv = size(A,1);
ne = nnz(A);

disp('Convesion to Metis format started')
fileName  = [fileName,'-metis-graph'];

fid = fopen(fileName,'w');
a = [nv ne/2];
str = mat2str(a);
str = str(2:end-1);
fprintf(fid,str,'char'); 
%fprintf(fid,'\n','char');
fprintf(fid,'\n');

for i = 1:nv
    a = find(A(:,i));
    str = mat2str(a');
    if size(a,1) > 1
        str = str(2:end-1);
    end
    fprintf(fid,str,'char');
    %fprintf(fid,'\n','char');
    fprintf(fid,'\n');
end
fclose(fid);
disp(['Metis graph saved into ', fileName])
