function  [msk,nnzBlFr,totFrac] = determineDenseBlocks(A,rprtc,cprtc,thrsh)
%--------------------------------------------------------------------------
% Function that determines blocks of A that are to be considered dense.
% 
% [msk,nnzBlFr,totFrc] = determineDenseBlocks(A,rprtc,cprtc,threshold)
%
% Input:
% A     - Sparse matrix 
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
% thrsh - Threshold value 
% 
% Output:
% msk     - Binary mask matrix indicating dense blocks
% nnzBlFr - Matrix with fraction of non-zeros in % for all blocks
% totFrac - Fraction of non-zeros within all dense blocks.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

%% Compute fraction of nonzero entries in each block. 
[~, nnzBlFr] = compClustNnzRect(A,rprtc,cprtc);

% Get all blocks that are above a given threshold
msk = nnzBlFr > thrsh;
csum = sum(msk,1);
rsum = sum(msk,2);

%% Make sure every block row and block column has at least one dense block.
for i = 1:length(csum)
    if csum(i) == 0
        maxVal = max(nnzBlFr(:,i));
        msk(:,i) = nnzBlFr(:,i) > .8*maxVal;
    end
end

for i = 1:length(rsum)
    if rsum(i) == 0
        maxVal = max(nnzBlFr(i,:));
        msk(i,:) = nnzBlFr(:,i) > .8*maxVal;
    end
end

totFrac = sum(sum(msk.*nnzBlFr));
