function B = matrixThreshold(A,tol)
%--------------------------------------------------------------------------
% Threshold matrix values, zero all entris with A(i,j) < tol.
% 
% B = matrixThreshold(A,tol);
% 
% Input:
% A     - Sparse or dense matrix 
% tol   - Tolarance, if A(i,j) < tol, then set B(i,j) = 0
% 
% Output:
% B     - Sparse matrix, of same dimensions as A, such that 
%         B(i,j) = A(i,j) if A(i,j) >= tol, or 
%         B(i,j) = 0 if A(i,j) < tol.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

[m,n] = size(A);
[rind,cind,val] = find(A);
ind = val >= tol;

B = sparse(rind(ind),cind(ind),val(ind),m,n);
