function mem = memUsageCmapp(U,S,V,str)
%--------------------------------------------------------------------------
% Memory usage in a clustered matrix approximation
%
% mem = memUsageCmapp(U,S,V,'non-sym');
% mem = memUsageCmapp(U,S,[],'sym');
%
% Input:
% U     - Cell-array with orthonormal matrices Ui
% S     - Matrix with Sij = Ui'*Aij*Vj
% V     - Cell-array with orthonormal matrices Vj
%         V is replaced with [] when the approximation is symmetric
% str   - string, must be 'sym' or 'non-sym'
%
% Output:
% mem   - Memory usage in number of floating point numbers
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------


mem = 0;

for i = 1:length(U)
    mem = mem + numel(U{i});
end

switch str
    case 'sym'
        n = size(S,1);
        mem = mem + n*(n-1)/2;
    case 'non-sym'
        mem = mem + numel(S);
        for i = 1:length(V)
            mem = mem + numel(V{i});
        end
    otherwise
        disp('str variable must be sym or non-sym')
end




