function [mem,k] = memUsageRmapp(m,n,memCmapp,str)
%--------------------------------------------------------------------------
% Memory usage in a regular matrix approximation and its rank k
%
% [mem,k] = memUsageRmapp(m, n,memCmapp,'non-sym');
% [mem,k] = memUsageRmapp(m,[],memCmapp,'sym');
%
% Input:
% m         - number of rows
% n         - number of columns
% memCmapp  - memory usage for clustered matrix approximation
% str       - string, must be 'sym' or 'non-sym'
%
% Output:
% mem   - Memory usage in number of floating point numbers
% k     - Rank of a regular matrix approximation
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

switch str
    case 'sym'
        % k in favour of regular matrix approximation
        k = ceil(memCmapp/(m+1));
        mem = (m+1)*k;
    case 'non-sym'
        % k in favour of regular matrix approximation
        k = ceil(memCmapp/(m+n+1));
        mem = (m+n+1)*k;
    otherwise
        disp('str variable must be sym or non-sym')
end




