function A = normalizeColumns(A)
%--------------------------------------------------------------------------
% Normalize columns of a sparse matrix A. 
% 
% A = normalizeColumns(A);
%
% Input:
% A     - Sparse matrix
%
% Output:
% A     - Columns of output matrix have norm equal to one. 
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

[m,n] = size(A);
nnzA = nnz(A);

rind = zeros(nnzA,1);
cind = zeros(nnzA,1);
vind = zeros(nnzA,1);

lowInd = 1;

for i = 1:n
    [rr,~,vv] = find(A(:,i));
    uppInd = lowInd + length(rr)-1;
    
    rind(lowInd:uppInd) = rr;
    cind(lowInd:uppInd) = i;
    vind(lowInd:uppInd) = vv/norm(vv);
    
    lowInd = uppInd + 1;    
end

A = sparse(rind,cind,vind,m,n);

