function Y = randAlg(A,k,p,str)
%--------------------------------------------------------------------------
% Implementation of Alg 4.1 in Halko et al. 2009, i.e., Y = A*Omega
% 
% Y = randAlg(A,k,p,str);
%
% Input:
% A     - matrix to compute low rank approximation of. 
% k     - target rank of the approximation.
% p     - oversampling parameter, should be less then 10 (or at most k).
% str   - string to indicate orthnormal output Y or not.
%           str = 'orth' to return orthonormal matrix.
%           str = 'no-orth' to skip the orthonormalization step.
%
% Output:
% Y     - matrix that approximates the dominat k+p dimensional range of A
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

Y = A*randn(size(A,2),k+p);

switch str
    case 'orth'
        disp('randAlg with orthonormaliztion')
        [Y, ~] = qr(Y,0);
    case 'no-orth'
        disp('randAlg without orthonormaliztion')
    otherwise
        disp('str must be a string and one of "orth" or "no-orth"')
end

