function Y = randAlgPow(A,k,p,q,str)
%--------------------------------------------------------------------------
% Randomized algorithm to compute an approximation to the dominant column
% space of a given matrix A. The output is Y = ( (A*A')^q )*A*Omega or an
% orthonormal basis for the range of Y.
%
% Y = randAlgPow(A,k,p,q,str)
%
% Input:
% A     - matrix to compute low rank approximation of
% k     - target rank of the approximation
% p     - oversampling parameter, should be less then 10 (or at most k)
% q     - number of power iterations, q is small, e.g. 2 or 3
% str   - string to indicate orthnormal output Y or not.
%           str = 'orth' to return orthonormal matrix
%           str = 'no-orth' to skip the last orthonormalization step
%
% Output:
% Y     - matrix that approximates the dominat k+p dimensional range of A
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

Y = randn(size(A,2),k+p);

Y = A*Y;
if q > 0
    for i = 1:q
        [Y,~] = qr(Y,0);
        Z = A'*Y;
        Y = A*Z;
    end
end

switch str
    case 'orth'
        disp('randAlgPow with orthonormaliztion')
        [Y,~] = qr(Y,0);
    case 'no-orth'
        disp('randAlgPow without orthonormaliztion')
    otherwise
        disp('str must be a string and one of "orth" or "no-orth"')
end




