function [prt,prtsz] = recGraclus(A,mcs)
%--------------------------------------------------------------------------
% Recursive graph partitioning using graclus
%
% [prt,prtsz] = recGraclus(A,mcs);
%
% Input:
% A     - Adjacency matrix of a graph
% mcs   - Maximum size of a cluster
%
% Output:
% prt   - Vector indicating cluster belonging for each vertex.
% prtsz - Vector indicating the size of each cluster.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

%% Main part
[prt, ~] = graclus(A,2);
[prt] = componentAnalysis(prt,A);

prtsz(1,1) = sum(prt == 1);
prtsz(2,1) = sum(prt == 2);

prtmore = true;
nprt = prt;         % new partition
nprtsz = prtsz;     % new partition sizes

while prtmore
    cnoc = max(prt);    % current number of clusters
    for i = 1:size(prtsz,1)
        if prtsz(i) > mcs
            % partition
            ind = find(prt == i);
            [tprt, ~] = graclus(A(ind,ind),2);
            
            % component analysis to make sure both of the clusters contain
            % a single component.
            [tprt] = componentAnalysis(tprt,A(ind,ind));
            
            ind1 = (tprt == 1);
            ind2 = (tprt == 2);
            
            tprt(ind1) = i;
            tprt(ind2) = cnoc +1;
            nprt(ind) = tprt;
            nprtsz(i) = sum(ind1);
            nprtsz(cnoc+1) = sum(ind2);
            cnoc = cnoc + 1;
        end
    end
    prt = nprt;
    prtsz = nprtsz;
    if max(prtsz) <= mcs
        prtmore = false ;
    end
end

function prt = componentAnalysis(prt,B)
%% 
ind1 = (prt == 1);
[ci,sz] = components(B(ind1,ind1));
[~, maxind] = max(sz);
indprt2 = (ci ~= maxind);
find1 = find(ind1);
prt(find1(indprt2)) = 2;

%%
ind2 = (prt == 2);
[ci,sz] = components(B(ind2,ind2));
[~, maxind] = max(sz);
indprt1 = (ci ~= maxind);
find2 = find(ind2);
prt(find2(indprt1)) = 1;



