function [ind,prtsz,prtc] = reorderVertices(prt)
%--------------------------------------------------------------------------
% Reorder vertices according to cluster belonging.
%
% [ind,prtsz,prtc] = reorderVertices(prt);
%
% Input:
% prt   - Vector with cluster index for each vertex
%
% Output:
% ind   - Vector reordered vertices according to cluster belonging.
%           Check clustering structure with: spy(A(ind,ind))
% prtsz - Vector with cluster sizes
% prtc  - Cell-array with each cell containing vertices for one
%           cluster. Extract the ij-block of the adjacency matrix with
%           A{i,j} = A(prtc{i},prtc{j});
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of clusters
noc = max(prt);

% initiate output variabels
ind     = zeros(size(prt));
prtsz   = zeros(noc,1);
prtc    = cell(noc,1);

k2 = 0;
for i = 1:noc
    prtc{i}  = find(prt == i);
    prtsz(i) = size(prtc{i},1);
    k1 = k2 + 1;
    k2 = k1 + prtsz(i) - 1;
    ind(k1:k2) = prtc{i};
end

